@@ -16,27 +16,28 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 **************************************************************************** */
 
 #include "brain.h"
-#include "cubebox.h"
 #include "cube.h"
-
+#include <QApplication>
 #include <math.h>
 
-#include <kapplication.h>
 
 #undef DEBUG // uncomment this to get useful messages
+#include <assert.h>
 
 #ifdef DEBUG
 #include <iostream.h>
 #endif
 
+#include "prefs.h"
+
 Brain::Brain(int initValue)
 {
-   setSkill(Beginner);
+   setSkill(Prefs::EnumSkill::Beginner);
    stopped=false;
    active=false;
    currentLevel=0;
@@ -45,30 +46,27 @@ Brain::Brain(int initValue)
    random.setSeed(initValue);
 }
 
-Brain::Skill Brain::setSkill(Skill newSkill)
+void Brain::setSkill(int newSkill)
 {
-   Skill oldSkill=_skill;
    _skill=newSkill;
 
    switch(_skill)
    {
-      case Beginner:
+      case Prefs::EnumSkill::Beginner:
          maxLevel=1;
 	 break;
-      case Average:
+      case Prefs::EnumSkill::Average:
          maxLevel=3;
          break;
-      case Expert:
+      case Prefs::EnumSkill::Expert:
          maxLevel=5;
          break;
       default:
          break;
    }
-
-   return oldSkill;
 }
 
-Brain::Skill Brain::skill() const
+int Brain::skill() const
 {
    return _skill;
 }
@@ -96,7 +94,7 @@ bool Brain::getHint(int& row, int& column,CubeBox::Player player ,CubeBox box)
    currentPlayer=player;
 
    int i=0,j=0;
-   int moves=0; // how many moves are the favourable ones
+   int moves=0; // how many moves are the favorable ones
    CubeBox::Player opponent=(player==CubeBox::One)?CubeBox::Two : CubeBox::One;
 
    // if more than one cube has the same rating this array is used to select
@@ -108,8 +106,8 @@ bool Brain::getHint(int& row, int& column,CubeBox::Player player ,CubeBox box)
    for(i=0;i<box.dim();i++)
       worth[i]=new double[box.dim()];
 
-   // alle Werte auf kleinstmglichen Wert setzen
-   double min=-pow(2.0,sizeof(long int)*8-1);  // Maximum auf kleinst mglichen Wert setzen
+   // alle Werte auf kleinstmöglichen Wert setzen
+   double min=-pow(2.0,sizeof(long int)*8.-1);  // Maximum auf kleinst möglichen Wert setzen
 
    for(i=0;i<box.dim();i++)
      for(j=0;j<box.dim();j++)
@@ -118,7 +116,7 @@ bool Brain::getHint(int& row, int& column,CubeBox::Player player ,CubeBox box)
      }
 
 
-   // find the favourable cubes to increase
+   // find the favorable cubes to increase
    moves=findCubes2Move(c2m,player,box);
 
 
@@ -126,7 +124,7 @@ bool Brain::getHint(int& row, int& column,CubeBox::Player player ,CubeBox box)
    if(moves==1)
    {
 #ifdef DEBUG
-      cerr << "found only one favourable cube" << endl;
+      cerr << "found only one favorable cube" << endl;
 #endif
       row=c2m[0].row;
       column=c2m[0].column;
@@ -134,24 +132,16 @@ bool Brain::getHint(int& row, int& column,CubeBox::Player player ,CubeBox box)
    else
    {
 #ifdef DEBUG
-      cerr << "found more than one favourable cube: " << moves << endl;
+      cerr << "found more than one favorable cube: " << moves << endl;
 #endif
       for(i=0;i<moves;i++)
       {
 	 // if Thinking process stopped
-	 if(stopped)
-	 {
+	 if (stopped) {
 #ifdef DEBUG
 	     cerr << "brain stopped" << endl;
 #endif
-	     active=false;
-             for(i=0;i<box.dim();i++)
-                delete[] worth[i];
-             delete [] worth;
-
-             delete [] c2m;
-
-	     return false;
+             break;	// Go evaluate the best move calculated so far.
 	 }
 
 #ifdef DEBUG
@@ -233,7 +223,7 @@ bool Brain::getHint(int& row, int& column,CubeBox::Player player ,CubeBox box)
 
    active=false;
 
-   return true;
+   return (! stopped);
 }
 
 
@@ -272,7 +262,7 @@ double Brain::doMove(int row, int column, CubeBox::Player player , CubeBox box)
       // the next move has does the other player
       player=(player==CubeBox::One)? CubeBox::Two : CubeBox::One;
 
-      // find the favourable cubes to increase
+      // find the favorable cubes to increase
       moves=findCubes2Move(c2m,player,box);
 
       // if only one cube is found, then don't check recursively the move
@@ -285,7 +275,7 @@ double Brain::doMove(int row, int column, CubeBox::Player player , CubeBox box)
       {
          for(i=0;i<moves;i++)
          {
-            kapp->processEvents();
+            qApp->processEvents();
 
 	    // if thinking process stopped
 	    if(stopped)
@@ -321,7 +311,7 @@ int Brain::findCubes2Move(coordinate *c2m,CubeBox::Player player,CubeBox& box)
    int moves=0;
    int min=9999;
 
-   if(_skill==Beginner)
+   if(_skill==Prefs::EnumSkill::Beginner)
    {
       int max=0;
       for(i=0;i<box.dim();i++)
@@ -380,7 +370,7 @@ int Brain::findCubes2Move(coordinate *c2m,CubeBox::Player player,CubeBox& box)
 	      if(currentLevel==0)
 	         cerr << i << "," << j << " : " << val << endl;
 #endif
-	      // only if val >= 0 its a favourable move
+	      // only if val >= 0 it is a favorable move
               if( val > 0 )
               {
 	         if(val<min)
@@ -430,7 +420,7 @@ int Brain::findCubes2Move(coordinate *c2m,CubeBox::Player player,CubeBox& box)
 
               counter++;
            }
-	   else if(_skill == Average)
+	   else if(_skill == Prefs::EnumSkill::Average)
 	   {
 	      if(c2m[i].val == secondMin)
 	      {
@@ -450,7 +440,7 @@ int Brain::findCubes2Move(coordinate *c2m,CubeBox::Player player,CubeBox& box)
    }
 
    int maxMoves=10;
-	// if more than maxMoves moves are favourable, take maxMoves random moves
+	// if more than maxMoves moves are favorable, take maxMoves random moves
 	// because it will take to much time if you check all
 	if(moves > maxMoves)
 	{
