@@ -34,7 +34,8 @@
 #include <QHeaderView>
 #include <QItemDelegate>
 #include <QDebug>
-#include "player.h"
+
+#include "gamelogic.h"
 #include "map.h"
 
 
@@ -58,14 +59,16 @@ static const QColor PlayerColors[MAX_PLAYERS] = {
 class playersListModel : public QAbstractTableModel
 {
     public:
-        playersListModel(QObject *parent) : QAbstractTableModel(parent)
+   playersListModel(QObject *parent, 
+                    QList<Player *> *players)
+      : QAbstractTableModel(parent), m_players(players)
         {
         }
 
         int rowCount(const QModelIndex &index = QModelIndex()) const
         {
             Q_UNUSED(index);
-            return m_players.count();
+            return m_players->count();
         }
 
         int columnCount(const QModelIndex&) const
@@ -81,22 +84,22 @@ class playersListModel : public QAbstractTableModel
                 int column = index.column();
                 if (role == Qt::DecorationRole && column == 0)
                 {
-                    return PlayerColors[row];
+                    return m_players->at(row)->color();
                 }
                 else if (role == Qt::DisplayRole)
                 {
                     if (column == 0)
                     {
-                        return m_players.at(row).first;
+                        return m_players->at(row)->name();
                     }
                     else if (column == 1)
                     {
-                        switch (m_players.at(row).second)
+                        switch (m_players->at(row)->getAiLevel())
                         {
-                            case Human: return i18nc("A human player", "Human"); break;
-                            case ComputerWeak: return i18n("Computer Weak"); break;
-                            case ComputerNormal: return i18n("Computer Normal"); break;
-                            case ComputerHard: return i18n("Computer Hard"); break;
+                            case Player::Human: return i18nc("A human player", "Human"); break;
+                            case Player::ComputerWeak: return i18n("Computer Weak"); break;
+                            case Player::ComputerNormal: return i18n("Computer Normal"); break;
+                            case Player::ComputerHard: return i18n("Computer Hard"); break;
                         }
                     }
                 }
@@ -114,7 +117,7 @@ class playersListModel : public QAbstractTableModel
             return QVariant();
         }
 
-        bool setData(const QModelIndex &index, const QVariant &value, int role)
+        bool setData(const QModelIndex &index, const QVariant &value, int)
         {
             bool result = false;
             if (index.isValid())
@@ -124,17 +127,20 @@ class playersListModel : public QAbstractTableModel
 
                 if (column == 0)
                 {
-                    m_players[row].first = value.toString();
+                    m_players->at(row)->setName(value.toString());
                     result = true;
                 }
                 else if (column == 1)
                 {
-qDebug() << "Setting the player type.";
                     QString text = value.toString();
-                    if (text == i18nc("A human player", "Human")) m_players[row].second = Human;
-                    else if (text == i18n("Computer Weak")) m_players[row].second = ComputerWeak;
-                    else if (text == i18n("Computer Normal")) m_players[row].second = ComputerNormal;
-                    else if (text == i18n("Computer Hard")) m_players[row].second = ComputerHard;
+                    if (text == i18nc("A human player", "Human"))
+                        m_players->at(row)->setAiLevel(Player::Human);
+                    else if (text == i18n("Computer Weak"))
+                        m_players->at(row)->setAiLevel(Player::ComputerWeak);
+                    else if (text == i18n("Computer Normal"))
+                        m_players->at(row)->setAiLevel(Player::ComputerNormal);
+                    else if (text == i18n("Computer Hard"))
+                        m_players->at(row)->setAiLevel(Player::ComputerHard);
                     result = true;
                 }
             }
@@ -144,9 +150,10 @@ qDebug() << "Setting the player type.";
             return result;
         }
 
-        void addPlayer()
+        Player *addPlayer(Map *map)
         {
-            int players = m_players.count();
+            Player *player = NULL;
+            int players = m_players->count();
             if (players < MAX_PLAYERS)
             {
                 bool invalidName = true;
@@ -155,16 +162,20 @@ qDebug() << "Setting the player type.";
                 while (invalidName) {
                     name = QString( i18nc("Default player name is \"player \" + player number", "Player %1", i) );
                     invalidName = false;
-                    for (int j = 0 ; !invalidName && j < m_players.count(); j++)
+                    for (int j = 0 ; !invalidName && j < m_players->count(); j++)
                     {
-                        invalidName = (m_players.at(j).first == name);
+                        invalidName = (m_players->at(j)->name() == name);
                     }
                     i++;
                 }
                 beginInsertRows(QModelIndex(), players, players);
-                m_players << QPair<QString, Player>( name, Human );
+
+                player = new Player( map, name, PlayerColors[i-2], i-1, Player::Human);
+                m_players->append(player);
+        
                 endInsertRows();
             }
+            return player;
         }
 
         Qt::ItemFlags flags(const QModelIndex &) const
@@ -172,31 +183,32 @@ qDebug() << "Setting the player type.";
             return Qt::ItemIsEnabled | Qt::ItemIsSelectable | Qt::ItemIsEditable;
         }
 
-        void removePlayer(int row)
+        Player *removePlayer(int row)
         {
-            if (row >= 0 && row < m_players.count())
+            Player *player = NULL;
+            if (row >= 0 && row < m_players->count())
             {
                 beginRemoveRows(QModelIndex(), row, row);
-                m_players.removeAt(row);
+                player = m_players->at(row);
+                m_players->removeAt(row);
                 endRemoveRows();
             }
+            return player;
         }
 
         bool hasHumans() const
         {
             bool humans = false;
-            for (int i = 0 ; !humans && i < m_players.count() ; ++i)
+            for (int i = 0 ; !humans && i < m_players->count() ; ++i)
             {
-                humans = (m_players.at(i).second == Human);
+                humans = (m_players->at(i)->getAiLevel() == Player::Human);
             }
             return humans;
         }
 
     private:
-        // TODO Move where it belongs
-        enum Player { Human, ComputerWeak, ComputerNormal, ComputerHard };
 
-        QList< QPair<QString, Player> > m_players;
+        QList<Player *> *m_players;
 };
 
 class playersListDelegate : public QItemDelegate
@@ -206,7 +218,7 @@ class playersListDelegate : public QItemDelegate
 		{
 		}
 		
-		QWidget * createEditor(QWidget *parent, const QStyleOptionViewItem &option, const QModelIndex &index) const
+		QWidget * createEditor(QWidget *parent, const QStyleOptionViewItem &, const QModelIndex &index) const
 		{
 			if (index.column() == 0)
 				return new KLineEdit(parent);
@@ -245,12 +257,12 @@ class playersListDelegate : public QItemDelegate
 };
 
 NewGameDlg::NewGameDlg( QWidget *parent, Map *pmap, QList<Player *> *players,
-                        Player *neutralPlayer, QList<Planet *> *planets )
+                        Player *neutralPlayer, GameOptions &options)
     : KDialog( parent),
       m_map(pmap),
       m_players(players),
-      m_planets(planets),
-      m_neutral(neutralPlayer)
+      m_neutral(neutralPlayer),
+      m_options(options)
 {
     setCaption(i18n("Start New Game"));
     setButtons(KDialog::Ok|KDialog::Cancel);
@@ -260,25 +272,42 @@ NewGameDlg::NewGameDlg( QWidget *parent, Map *pmap, QList<Player *> *players,
     m_w = new NewGameDlgUI(this);
     m_w->map->setMap( m_map );
 
-    playersListModel *model = new playersListModel(this);
+    connect(m_map, SIGNAL(selectionChanged(Sector*)),
+            this, SLOT(slotUpdateSelection(Sector*)));
+
+    playersListModel *model = new playersListModel(this, m_players);
 
     m_w->playerList->setModel(model);
     m_w->playerList->setItemDelegate(new playersListDelegate(this));
     m_w->playerList->header()->setResizeMode(QHeaderView::Stretch);
     
-    connect(m_w->neutralPlanetsSB, SIGNAL(valueChanged(int)), this, SLOT(slotNewMap()));
+    connect(m_w->neutralPlanetsSB, SIGNAL(valueChanged(int)), this, SLOT(slotUpdateNeutrals(int)));
     connect(m_w->widthSB, SIGNAL(valueChanged(int)), this, SLOT(slotNewMap()));
     connect(m_w->heightSB, SIGNAL(valueChanged(int)), this, SLOT(slotNewMap()));
-    connect(m_w->rejectMap, SIGNAL(clicked()), this, SLOT(slotNewMap()));
+    connect(m_w->randomizeMap, SIGNAL(clicked()), this, SLOT(slotNewMap()));
     connect(m_w->addPlayerButton, SIGNAL(clicked()), this, SLOT(slotAddPlayer()));
     connect(m_w->removePlayerButton, SIGNAL(clicked()), this, SLOT(slotRemovePlayer()));
-    connect(model, SIGNAL(dataChanged (const QModelIndex &, const QModelIndex &)), this, SLOT(slotNewMap()));
+
+    connect(m_w->OwnerCB, SIGNAL(currentIndexChanged(int)), this, SLOT(slotNewOwner(int)));
+    connect(m_w->KillPercentageSB, SIGNAL(valueChanged(double)), this, SLOT(slotNewKillPercentage(double)));
+    connect(m_w->ProductionSB, SIGNAL(valueChanged(int)), this, SLOT(slotNewProduction(int)));
+
     init();
 
     setMainWidget(m_w);
     slotNewMap();
 }
 
+void
+NewGameDlg::updateOwnerCB()
+{
+    m_w->OwnerCB->clear();
+
+    m_w->OwnerCB->addItem("vacant");
+    foreach(Player *player, *m_players)
+        m_w->OwnerCB->addItem(player->name());
+    m_w->OwnerCB->addItem("neutral");
+}
 
 void
 NewGameDlg::init()
@@ -299,6 +328,18 @@ NewGameDlg::init()
     m_w->widthSB->setValue(sizeWid);
     m_w->heightSB->setValue(sizeHeight);
     
+    m_w->BlindMapCB->setCheckState(
+       config.readEntry("BlindMap", false) ? Qt::Checked : Qt::Unchecked);
+    m_w->CumulativeProductionCB->setCheckState(
+       config.readEntry("CumulativeProduction", false) ? Qt::Checked : Qt::Unchecked);
+    m_w->ProductionAfterConquereCB->setCheckState(
+       config.readEntry("ProductionAfterConquere", true) ? Qt::Checked : Qt::Unchecked);
+    m_w->NeutralsShowShipsCB->setCheckState(
+       config.readEntry("NeutralsShowShips", false) ? Qt::Checked : Qt::Unchecked);
+    m_w->NeutralsShowStatsCB->setCheckState(
+       config.readEntry("NeutralsShowStats", false) ? Qt::Checked : Qt::Unchecked);
+    m_w->NeutralsProductionSB->setValue(config.readEntry("NeutralsProduction", 1));
+
     // Restore player names
     playersListModel *model = static_cast<playersListModel*>(m_w->playerList->model());
     for(int i = 0; i < nrOfPlayers; ++i)
@@ -309,28 +350,41 @@ NewGameDlg::init()
        QString playerName = config.readEntry(keyName,QString());
        QString playerType = config.readEntry(keyType,QString());
        
-       model->addPlayer();
+       model->addPlayer(m_map);
        if (!playerName.isEmpty()) model->setData(model->index(i, 0), playerName, Qt::EditRole);
        model->setData(model->index(i, 1), playerType, Qt::EditRole);
     }
+   updateOwnerCB();
 }
 
 void
 NewGameDlg::slotAddPlayer()
 {
     playersListModel *model = static_cast<playersListModel*>(m_w->playerList->model());
-    model->addPlayer();
 
-    slotNewMap();
+    Player *player = model->addPlayer(m_map);
+    if(player)
+        m_map->addPlayerPlanetSomewhere(player);
+
+    enableButtonOk(model->rowCount() > 0);
+
+    updateOwnerCB();
 }
 
 void
 NewGameDlg::slotRemovePlayer()
 {
     playersListModel *model = static_cast<playersListModel*>(m_w->playerList->model());
-    model->removePlayer(m_w->playerList->currentIndex().row());
 
-    slotNewMap();
+    Player *player = model->removePlayer(m_w->playerList->currentIndex().row());
+    if(player) {
+        m_map->removePlayerPlanets(player);
+        delete player;
+    }
+
+    enableButtonOk(model->rowCount() > 0);
+
+    updateOwnerCB();
 }
 
 void
@@ -348,6 +402,67 @@ NewGameDlg::slotOk()
     KDialog::accept();
 }
 
+void
+NewGameDlg::slotUpdateSelection(Sector *selected)
+{
+    bool planet = selected->hasPlanet();
+
+    m_w->KillPercentageSB->setEnabled(planet);
+    m_w->ProductionSB->setEnabled(planet);
+
+    disconnect(m_w->OwnerCB, SIGNAL(currentIndexChanged(int)), this, SLOT(slotNewOwner(int)));
+
+    if(planet) {
+        Planet *p = selected->planet();
+        int f = m_w->OwnerCB->findText(p->player()->name());
+        if(f>=0)
+            m_w->OwnerCB->setCurrentIndex(f);
+        else
+            m_w->OwnerCB->setCurrentIndex(m_w->OwnerCB->count()-1);
+
+
+        m_w->KillPercentageSB->setValue(p->killPercentage());
+        m_w->ProductionSB->setValue(p->production());
+    } else
+        m_w->OwnerCB->setCurrentIndex(0);
+
+    connect(m_w->OwnerCB, SIGNAL(currentIndexChanged(int)), this, SLOT(slotNewOwner(int)));
+}
+void
+NewGameDlg::slotNewOwner(int index)
+{
+    Sector *selected = m_map->sector(m_map->selectedSector());
+    if(selected->hasPlanet())
+        delete selected->planet();
+
+    if(index)
+        m_map->addPlanet(selected,
+                         index - 1 < m_players->length()
+                         ? (*m_players)[index-1]
+                         :  m_neutral,
+                         m_w->ProductionSB->value(),
+                         m_w->KillPercentageSB->value());
+
+    m_w->KillPercentageSB->setEnabled(index);
+    m_w->ProductionSB->setEnabled(index);
+}
+
+void
+NewGameDlg::slotNewKillPercentage(double value)
+{
+    Sector *selected = m_map->sector(m_map->selectedSector());
+    if(selected->hasPlanet())
+        selected->planet()->setKillPercentage(value);
+}
+
+void
+NewGameDlg::slotNewProduction(int value)
+{
+    Sector *selected = m_map->sector(m_map->selectedSector());
+    if(selected->hasPlanet())
+        selected->planet()->setProduction(value);
+}
+
 void
 NewGameDlg::save()
 {
@@ -357,6 +472,19 @@ NewGameDlg::save()
     config.writeEntry("SizeWidth", m_w->widthSB->value());
     config.writeEntry("SizeHeight", m_w->heightSB->value());
 
+#define CONFIG_WRITE_ENTRY_CB(NAME) \
+    config.writeEntry(#NAME, m_options.NAME \
+                      = m_w->NAME##CB->checkState() == Qt::Checked);
+
+    CONFIG_WRITE_ENTRY_CB(BlindMap);
+    CONFIG_WRITE_ENTRY_CB(CumulativeProduction);
+    CONFIG_WRITE_ENTRY_CB(ProductionAfterConquere);
+    CONFIG_WRITE_ENTRY_CB(NeutralsShowShips);
+    CONFIG_WRITE_ENTRY_CB(NeutralsShowStats);
+
+    config.writeEntry("NeutralsProduction",
+                      m_options.NeutralsProduction = m_w->NeutralsProductionSB->value());
+
     playersListModel *model = static_cast<playersListModel*>(m_w->playerList->model());
     config.writeEntry("NrOfPlayers", model->rowCount());
     for (int i = 0; i < model->rowCount(); ++i)
@@ -378,31 +506,32 @@ NewGameDlg::slotNewMap()
     // Clear map, player and planet lists
     m_map->clearMap();
 
-    while (!m_planets->isEmpty())
-        delete m_planets->takeFirst();
-
     m_map->resizeMap(m_w->heightSB->value(), m_w->widthSB->value());
 
-    while (!m_players->isEmpty())
-        delete m_players->takeFirst();
-
-    // Make player list
-    // Does the name already exist in the list
-    playersListModel *model = static_cast<playersListModel*>(m_w->playerList->model());
-    for (int i = 0; i < model->rowCount(); ++i) {
-        QString  playerName = model->data(model->index(i, 0), Qt::DisplayRole).toString();
-        QColor  color = model->data(model->index(i, 0), Qt::DecorationRole).value<QColor>();
-
-        // TODO: This is a bit ugly, maybe a isAI in model will be better.
-        bool ai = model->data( model->index(i, 1), Qt::DisplayRole).toString() != i18nc("A human player", "Human");
-        
-        // TODO: This is not going to work as changing names/player
-        //       type in the table needs to change name in m_players 
-        //       without changing the map.
-        m_players->append( Player::createPlayer( m_map, playerName, color, i, ai ));
+    // make sure we have space for neutrals
+    int maxneutrals = m_w->heightSB->value()*m_w->widthSB->value() - m_players->count();
+    if(m_w->neutralPlanetsSB->value() > maxneutrals) {
+        disconnect(m_w->neutralPlanetsSB, SIGNAL(valueChanged(int)), this, SLOT(slotNewMap()));
+        m_w->neutralPlanetsSB->setValue(maxneutrals);
+        connect(m_w->neutralPlanetsSB, SIGNAL(valueChanged(int)), this, SLOT(slotNewMap()));
     }
+
     // make the planets
-    m_map->populateMap( *m_players, m_neutral,
-            m_w->neutralPlanetsSB->value(),
-            *m_planets );
+    m_map->populateMap( *m_players, m_neutral, m_w->neutralPlanetsSB->value());
+
+    if(m_map->hasSelectedSector())
+        slotUpdateSelection(m_map->sector(m_map->selectedSector()));
+}
+
+void
+NewGameDlg::slotUpdateNeutrals(int count)
+{
+   while(m_map->playerPlanetCount(m_neutral) < count
+         && m_map->addNeutralPlanetSomewhere(m_neutral)) ;
+
+   while(m_map->playerPlanetCount(m_neutral) > count
+         && m_map->removePlayerPlanet(m_neutral)) ;
+
+    if(m_map->hasSelectedSector())
+       slotUpdateSelection(m_map->sector(m_map->selectedSector()));
 }
