@@ -39,33 +39,33 @@
 #define GRANDF_H
 
 #include "dealer.h"
-class Pile;
 
 
 class Grandf : public DealerScene
 {
-    friend class GrandfSolver;
     Q_OBJECT
 
 public:
-    Grandf( );
-
-public slots:
-    void deal();
-    virtual void restart();
-    virtual Card *newCards();
+    virtual void initialize();
 
 protected:
-    void collect();
-    virtual bool checkAdd   ( int checkIndex, const Pile *c1, const CardList& c2) const;
-    virtual QString getGameState();
     virtual void setGameState( const QString & stream );
+    virtual QString getGameState();
+    virtual bool checkAdd(const PatPile * pile, const QList<KCard*> & oldCards, const QList<KCard*> & newCards) const;
+    virtual bool checkRemove(const PatPile * pile, const QList<KCard*> & cards) const;
+    virtual void restart();
+
+protected slots:
+    virtual bool newCards();
 
 private:
-    Pile* store[7];
-    Pile* target[4];
+    void deal( const QList<KCard*> & cardsToDeal );
+
+    PatPile* store[7];
+    PatPile* target[4];
     int numberOfDeals;
 
+    friend class GrandfSolver;
 };
 
 #endif
