@@ -3,7 +3,7 @@
  *   Copyright 1997  Mario Weilguni <mweilguni@sime.com>                   *
  *   Copyright 2002-2004  Dave Corrie <kde@davecorrie.com>                 *
  *   Copyright 2007  Mauricio Piacentini <mauricio@tabuleiro.com>          *
- *   Copyright 2009  Frederik Schwarzer <schwarzerf@gmail.com>             *
+ *   Copyright 2009,2010  Frederik Schwarzer <schwarzerf@gmail.com>        *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -159,7 +159,6 @@ public:
 
     void setSize(int x, int y);
     void resizeBoard();
-    void newGame();
     void setShuffle(int);
     int  shuffle() const;
 
@@ -226,6 +225,7 @@ public:
 
 signals:
     void markMatched(); // unused?
+    void newGameStarted();
     void changed();
     void endOfGame();
     void resized();
@@ -237,6 +237,13 @@ signals:
     void cheatStatusChanged();
 
 public slots:
+    /** Does most of the newGame work.
+     * This slot is called from the App::invokeNewGame() signal from App and
+     * should call App::newGame again to do the work that cannot be done
+     * from Board.
+     */
+    void newGame();
+
     /// Controls the pause mode
     void setPauseEnabled(bool enabled);
 
@@ -334,6 +341,7 @@ private: // functions
     void drawConnection(int timeout);
     void drawPossibleMoves(bool b);
     QPoint midCoord(int x, int y) const;
+    void unmarkTile();
     void marked(int x, int y);
     void madeMove(int x1, int y1, int x2, int y2);
     void madeMoveWithSlide(int x1, int y1, int x2, int y2, Path &slide);
