@@ -19,8 +19,11 @@
 // application specific includes
 #include "kgamewin.h"
 #include "ksirksettings.h"
+#include "newgamesetup.h"
 #include "Sprites/backgnd.h"
 #include "Sprites/arrowsprite.h"
+#include "Dialogs/newGameSummaryWidget.h"
+#include "GameLogic/newplayerdata.h"
 
 #include <kaboutapplicationdialog.h>
 #include <ktoolbar.h>
@@ -33,6 +36,7 @@
 #include "SaveLoad/ksirkgamexmlloader.h"
 #include "Sprites/animspritesgroup.h"
 #include "Dialogs/jabbergameui.h"
+#include "Dialogs/kplayersetupwidget.h"
 #include "Jabber/kmessagejabber.h"
 
 #include "kgame/kmessageserver.h"
@@ -58,6 +62,7 @@
 #include <kgamepopupitem.h>
 #include <KPasswordDialog>
 #include <KInputDialog>
+#include <tcpconnectwidget.h>
 
 namespace Ksirk
 {
@@ -399,6 +404,7 @@ void KGameWindow::slotArena(bool isCheck)
 
 void KGameWindow::slotJabberGame()
 {
+  m_jabberGameWidget->init(m_automaton);
   m_jabberGameWidget->setPreviousGuiIndex(m_centralWidget->currentIndex());
   m_centralWidget->setCurrentIndex(JABBERGAME_INDEX);
 }
@@ -406,15 +412,7 @@ void KGameWindow::slotJabberGame()
 void KGameWindow::slotNewGame()
 {
   //   kDebug() << "Slot new game: posting event actionNewGame";
-  //   QPoint point;
   actionNewGame(GameAutomaton::None);
-  
-  /// @TODO set the state to init when new game is started
-  /*  if (actionNewGame())
-  {
-    m_automaton->state(GameAutomaton::INIT);
-}*/
-//   m_automaton->gameEvent("actionNewGame", point);
 }
 
 void KGameWindow::slotNewJabberGame()
@@ -477,7 +475,10 @@ void KGameWindow::slotSaveGame()
     }
     if (!m_fileName.isEmpty())
     {
-      std::ofstream ofs(m_fileName.toUtf8().data());
+      QFile file(m_fileName);
+      file.open(QIODevice::WriteOnly);
+      QTextStream ofs(&file);
+      ofs.setCodec("UTF-8");
       saveXml(ofs);
       m_message->setMessageTimeout(3000);
       m_message->showMessage(i18n("Game saved to %1",m_fileName),
@@ -632,6 +633,25 @@ void KGameWindow::slotJoinNetworkGame()
   m_automaton->gameEvent("actionJoinNetworkGame", point);
 }
 
+void KGameWindow::slotConnectToServer()
+{
+  kDebug();
+  m_newGameSetup->setHost(m_tcpConnectWidget->hostEdit->text());
+  m_newGameSetup->setTcpPort(m_tcpConnectWidget->portEdit->value());
+  m_automaton->setGameStatus(KGame::End);
+  m_reinitializingGame = true;
+  
+  if (!(m_automaton->playerList()->isEmpty()))
+  {
+    m_automaton->playerList()->clear();
+    m_automaton->currentPlayer(0);
+    kDebug() << "  playerList size = " << m_automaton->playerList()->count();
+  }
+  theWorld()->reset();
+  
+  m_automaton->connectToServ();
+}
+
 void KGameWindow::slotShowGoal()
 {
   kDebug();
@@ -844,28 +864,40 @@ void KGameWindow::slotWindowDef2()
   m_defenseDialog->close();
 }
 
-void KGameWindow::slotNewGameOK(unsigned int nbPlayers, const QString& skin, unsigned int nbNetworkPlayers, bool useGoals)
+void KGameWindow::slotNewGameNext()
 {
-  kDebug() << nbPlayers << skin << nbNetworkPlayers << useGoals;
-  m_automaton->setGameStatus(KGame::End);
-  m_reinitializingGame = true;
-  m_automaton->removeAllPlayers();
-
-  showMap();
-  m_newPlayersNumber = nbPlayers;
-  m_automaton->setUseGoals(useGoals);
-  m_automaton->state(GameLogic::GameAutomaton::INIT);
-  m_automaton->savedState(GameLogic::GameAutomaton::INVALID);
-  m_automaton->setNetworkPlayersNumber(m_automaton->networkGameType()==GameAutomaton::None?0:nbNetworkPlayers);
-  m_automaton->finishSetupPlayersNumberAndSkin(skin, m_automaton->networkGameType(), nbPlayers);
+  kDebug();
+  m_automaton->newGameNext();
   m_reinitializingGame = false;
-
+  
   if (m_automaton->networkGameType()==GameAutomaton::Jabber && m_jabberClient && m_jabberClient->isConnected())
   {
     sendGameInfoToJabber();
   }
 }
 
+// void KGameWindow::slotNewGameOK(unsigned int nbPlayers, const QString& skin, unsigned int nbNetworkPlayers, bool useGoals)
+// {
+//   kDebug() << nbPlayers << skin << nbNetworkPlayers << useGoals;
+//   m_automaton->setGameStatus(KGame::End);
+//   m_reinitializingGame = true;
+//   m_automaton->removeAllPlayers();
+// 
+//   showMap();
+//   m_newPlayersNumber = nbPlayers;
+//   m_automaton->setUseGoals(useGoals);
+//   m_automaton->state(GameLogic::GameAutomaton::INIT);
+//   m_automaton->savedState(GameLogic::GameAutomaton::INVALID);
+//   m_automaton->setNetworkPlayersNumber(m_automaton->networkGameType()==GameAutomaton::None?0:nbNetworkPlayers);
+//   m_automaton->finishSetupPlayersNumberAndSkin(skin, nbPlayers);
+//   m_reinitializingGame = false;
+// 
+//   if (m_automaton->networkGameType()==GameAutomaton::Jabber && m_jabberClient && m_jabberClient->isConnected())
+//   {
+//     sendGameInfoToJabber();
+//   }
+// }
+
 void KGameWindow::slotNewGameKO()
 {
   kDebug() << m_stateBeforeNewGame << m_stackWidgetBeforeNewGame;
@@ -1111,5 +1143,101 @@ void KGameWindow::slotExit()
   close();
 }
 
+void KGameWindow::slotNewPlayerNext()
+{
+  kDebug() << m_newGameSetup->nbLocalPlayers() << m_newGameSetup->nbPlayers() << m_newGameSetup->nbNetworkPlayers();
+  if (m_automaton->isAdmin() && m_newGameSetup->nbLocalPlayers() >= m_newGameSetup->nbPlayers()-m_newGameSetup->nbNetworkPlayers())
+  {
+    m_newGameSummaryWidget->show(this);
+    m_centralWidget->setCurrentIndex(NEWGAMESUMMARY_INDEX);
+  }
+  else if (!m_automaton->isAdmin())
+  {
+    NewPlayerData* pd = m_newGameSetup->players().back();
+    addPlayer(pd->name(), 0, 0, pd->nation(), pd->computer());
+  }
+}
+
+void KGameWindow::slotNewPlayerPrevious()
+{
+  kDebug();
+  if (m_newGameSetup->players().empty())
+  {
+    m_centralWidget->setCurrentIndex(NEWGAME_INDEX);
+  }
+  else
+  {
+    NewPlayerData* player = m_newGameSetup->players().back();
+    m_newGameSetup->players().pop_back();
+    m_newPlayerWidget->init(player);
+    delete player;
+    m_centralWidget->setCurrentIndex(NEWPLAYER_INDEX);
+  }
+}
+
+void KGameWindow::slotNewPlayerCancel()
+{
+  kDebug();
+  /// @TODO other uninits to do
+  foreach (NewPlayerData* player, m_newGameSetup->players())
+  {
+    delete player;
+  }
+  m_newGameSetup->players().clear();
+  m_centralWidget->setCurrentIndex(m_stackWidgetBeforeNewGame);
+}
+
+void KGameWindow::slotStartNewGame()
+{
+  kDebug() << m_newGameSetup->nbPlayers() << m_newGameSetup->nbPlayers() << m_newGameSetup->players().size();
+  m_automaton->setGameStatus(KGame::End);
+  m_reinitializingGame = true;
+  
+  // for network games, remote players are already created, should not remove them
+  // TODO the players will not be in the orderd showed in the interface.
+  if (!(m_automaton->playerList()->isEmpty()) && m_automaton->networkGameType() == GameAutomaton::None)
+  {
+    kDebug() << "There was " << m_automaton->playerList()->count() << "players";
+    m_automaton->playerList()->clear();
+    m_automaton->currentPlayer(0);
+    kDebug() << "  playerList size = " << m_automaton->playerList()->count();
+  }
+  theWorld()->reset();
+  
+  m_automaton->finishSetupPlayersNumberAndSkin();
+  m_reinitializingGame = false;
+
+  unsigned int nbAvailArmies = (unsigned int)(m_theWorld->getNbCountries() * 2.5 / m_newGameSetup->players().size());
+  foreach (const NewPlayerData* player, m_newGameSetup->players())
+  {
+    if (!player->network())
+    {
+      // this has to be done locally
+      addPlayer(player->name(), nbAvailArmies, 0, player->nation(), player->computer());
+    }
+  }
+
+  if (m_newGameSetup->players().size() == m_newGameSetup->nbPlayers())
+  {
+    showMap();
+    m_frame->setFocus();
+  }
+  else
+  {
+    m_newGameSummaryWidget->finishButton->setDisabled(true);
+    m_newGameSummaryWidget->previousButton->setDisabled(true);
+  }
+}
+
+void KGameWindow::slotTcpConnectCancel()
+{
+  showMainMenu();
+}
+
+void KGameWindow::slotTcpConnectPrevious()
+{
+  showMainMenu();
+}
+
 
 } // closing namespace Ksirk
